# Generated by roxygen2: do not edit by hand

export(calcLocalTrapDists)
export(calcLocalTrapExposure)
export(calcWindowSizes)
export(calculateDensity)
export(dDispersal_exp)
export(dHabitatMask)
export(dbernppAC)
export(dbernppACmovement_exp)
export(dbernppACmovement_normal)
export(dbernppDetection_normal)
export(dbernppLocalACmovement_exp)
export(dbernppLocalACmovement_normal)
export(dbernppLocalDetection_normal)
export(dbinomLocal_exp)
export(dbinomLocal_normal)
export(dbinomLocal_normalPlateau)
export(dbinom_vector)
export(dcatState1Alive1Dead)
export(dcatState1Alive2Dead)
export(dcatState2Alive2Dead)
export(dmultiLocal_normal)
export(dnormalizer)
export(dpoisLocal_normal)
export(dpoisppAC)
export(dpoisppDetection_normal)
export(dpoisppLocalDetection_normal)
export(findLocalTraps)
export(getHomeRangeArea)
export(getLocalObjects)
export(getLocalTrapIndices)
export(getMidPointNodes)
export(getNumLocalTraps)
export(getSparseY)
export(getWindowCoords)
export(getWindowIndex)
export(integrateIntensityLocal_exp)
export(integrateIntensityLocal_normal)
export(integrateIntensity_exp)
export(integrateIntensity_normal)
export(makeConstantNimbleFunction)
export(makeGrid)
export(marginalVoidProbIntegrand)
export(marginalVoidProbNumIntegration)
export(rDispersal_exp)
export(rHabitatMask)
export(rbernppAC)
export(rbernppACmovement_exp)
export(rbernppACmovement_normal)
export(rbernppDetection_normal)
export(rbernppLocalACmovement_exp)
export(rbernppLocalACmovement_normal)
export(rbernppLocalDetection_normal)
export(rbinomLocal_exp)
export(rbinomLocal_normal)
export(rbinomLocal_normalPlateau)
export(rbinom_vector)
export(rcatState1Alive1Dead)
export(rcatState1Alive2Dead)
export(rcatState2Alive2Dead)
export(rmultiLocal_normal)
export(rnormalizer)
export(rpoisLocal_normal)
export(rpoisppAC)
export(rpoisppDetection_normal)
export(rpoisppLocalDetection_normal)
export(sampler_categorical_general)
export(scaleCoordsToHabitatGrid)
export(stratRejectionSampler_exp)
export(stratRejectionSampler_normal)
import(nimble)
importFrom(grDevices,adjustcolor)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(stats,dbinom)
importFrom(stats,dexp)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,pnorm)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,rpois)
importFrom(stats,runif)
